/* gingerのポート設定テーブル */

/* 各configurationの入出力ポート数 */
/*                                 0  1  2  3  4  5   6   7  8  9  X */
static volatile uchar DIPorts[] = {0, 4, 0, 4, 0, 16, 0,  0, 0, 0, 8};
static volatile uchar DOPorts[] = {0, 4, 4, 0, 0, 0, 16, 16, 0, 0, 8};
static volatile uchar AIPorts[] = {0, 4, 6, 4, 6, 0,  0,  0, 0, 0, 0};
static volatile uchar AOPorts[] = {0, 4, 4, 6, 6, 0,  0,  0, 0, 0, 0};

/* 入出力ピンのマッピングテーブル */
static struct portInfo din2[] = { /* for config 2,4 */
  {&PINB,0},{&PIND,1}};
static struct portInfo din4[] = { /* for config 1, 3 */
  {&PINC,4},{&PINC,5},{&PINB,0},{&PIND,1}};
static struct portInfo din8[] = { /* for config X */
  {&PINC,0},{&PINC,1},{&PINC,2},{&PINC,3},
  {&PINC,4},{&PINC,5},{&PINB,0},{&PIND,1}};
static struct portInfo din16[] = { /* for config 5 */
  {&PINC,0},{&PINC,1},{&PINC,2},{&PINC,3},
  {&PINC,4},{&PINC,5},{&PINB,0},{&PIND,1},
  {&PIND,6},{&PIND,5},{&PINB,1},{&PINB,2},
  {&PINB,3},{&PIND,3},{&PIND,4},{&PIND,7}};

static struct portInfo dout2[] = { /* for config 3, 4 */
  {&PORTD,4},{&PORTD,7}};
static struct portInfo dout4[] = { /* for config 1, 2 */
  {&PORTB,3},{&PORTD,3},{&PORTD,4},{&PORTD,7}};
static struct portInfo dout8[] = { /* for config X */
  {&PORTD,6},{&PORTD,5},{&PORTB,1},{&PORTB,2},
  {&PORTB,3},{&PORTD,3},{&PORTD,4},{&PORTD,7}};
static struct portInfo dout16[] = { /* for config 6,7 */
  {&PORTC,0},{&PORTC,1},{&PORTC,2},{&PORTC,3},
  {&PORTC,4},{&PORTC,5},{&PORTB,0},{&PORTD,1},
  {&PORTD,6},{&PORTD,5},{&PORTB,1},{&PORTB,2},
  {&PORTB,3},{&PORTD,3},{&PORTD,4},{&PORTD,7}};

/* Ainの情報は不要 */
static struct portInfo ain4[] = { /* for config 1,3 */
  {&PORTC,0},{&PORTC,1},{&PORTC,2},{&PORTC,3}};
static struct portInfo ain6[] = { /* for config 2,4 */
  {&PORTC,0},{&PORTC,1},{&PORTC,2},{&PORTC,3},
  {&PORTC,4},{&PORTC,5}};

static struct portInfo aout4[] = { /* for config 1,2 */
  {&OCR0A,0},{&OCR0B,0},{&OCR1AL,0},{&OCR1BL,0}};
static struct portInfo aout6[] = { /* for config 3,4 */
  {&OCR0A,0},{&OCR0B,0},{&OCR1AL,0},{&OCR1BL,0}, /* OCR1A,OCR1Bは16ビット */
  {&OCR2A,0},{&OCR2B,0}};
    
static volatile struct portMap portMapper[] = {
  {0,0,0,0},			/* config 0 (dummy) */
  {din4, dout4, ain4, aout4},	/* config 1 */
  {din2, dout4, ain6, aout4},	/* config 2 */
  {din4, dout2, ain4, aout6},	/* config 3 */
  {din2, dout2, ain6, aout6},	/* config 4 */
  {din16, 0, 0, 0},		/* config 5 */
  {0, dout16, 0, 0},		/* config 6 */
  {0, dout16, 0, 0},		/* config 7 */
  {0, 0, 0, 0},			/* config 8 dummy */
  {0, 0, 0, 0},			/* config 9 dummy */
  {din8, dout8, 0, 0,}		/* config X */
};

static portConfig portConf[] = {
  { 0x1f, 0x20, 0x3f, 0x00, 0xfa, 0x00}, /* Config 0 */
  { 0x11, 0x2e, 0x3f, 0x00, 0x02, 0xf8}, /* Config 1 */
  { 0x11, 0x2e, 0x3f, 0x00, 0x02, 0xf8}, /* Config 2 */
  { 0x11, 0x2e, 0x3f, 0x00, 0x02, 0xf8}, /* Config 3 */
  { 0x11, 0x2e, 0x3f, 0x00, 0x02, 0xf8}, /* Config 4 */
  { 0x1f, 0x20, 0x3f, 0x00, 0xfa, 0x00}, /* Config 5 */
  { 0x10, 0x2f, 0x00, 0xfa, 0x00, 0xfa}, /* Config 6 */
  { 0x10, 0x2f, 0x00, 0x3f, 0x00, 0xfa}, /* Config 7 */
  { 0x1f, 0x20, 0x3f, 0x00, 0xfa, 0x00}, /* Config 8 NOT USED */
  { 0x1f, 0x20, 0x3f, 0x00, 0xfa, 0x00}, /* Config 9 NOT USED*/
  { 0x11, 0x2e, 0x3f, 0x00, 0x02, 0xf8}	 /* Config X */
};
