/* gio.c */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>   /* needed by usbdrv.h */
#include "oddebug.h"
#include "gio.h"
#include "usbdrv.h"

/* gio buffer */
uchar    rx_buf[RX_SIZE], tx_buf[TX_SIZE];
uchar    urptr, uwptr, iwptr;

uchar verbose = 0;	/* verbose mode (?) */
uchar config = 0;	/* current configuration */
uchar pollMode = POLL_NONE; /* polling mode = off */

/* AD変換用 */
char convertingCh = CONVERTING_CH_NONE;	/* 変換中(変換中のチャネル番号） */

uchar smode = 0;

void gioInit(uchar conf) {
  urptr = uwptr = iwptr = 0;
  gioPortInit(conf);
  config = conf;
  verbose = 0;
  pollMode = POLL_NONE;
}

void gioRxBufAppendString(uchar *s) {
  uchar c;
  while ( (c = *s++)!= '\0') {
    gioRxBufAppend(c);
  }
}

#ifdef MODEL_GINGER
#include "ginger_pconfig.c"
#endif
#ifdef MODEL_SUGAR
#include "sugar_pconfig.c"
#endif


/* 全ポートを初期化する */
void gioPortInit(uchar conf) {
  config = conf;

#ifdef MODEL_SUGAR
  /* setup PORTA */
  DDRA &= ~portConf[conf].ddraIn;
  DDRA |= portConf[conf].ddraOut;
  PORTA |= portConf[conf].ddraIn; /* pull-up all input ports */
#endif

  /* setup PORTB */
  DDRB &= ~portConf[conf].ddrbIn;
  DDRB |= portConf[conf].ddrbOut;
  PORTB |= portConf[conf].ddrbIn; /* pull-up all input ports */

  /* setup PORTC */
  DDRC &= ~portConf[conf].ddrcIn;
  DDRC |= portConf[conf].ddrcOut;
  PORTC |= portConf[conf].ddrcIn; /* pull-up all input ports*/

  /* setup PORTD */
  DDRD &= ~portConf[conf].ddrdIn;
  DDRD |= portConf[conf].ddrdOut;
  PORTD |= portConf[conf].ddrdIn; /* pull-up all input ports */

  /* PWMの設定をする */
  if (AOPorts[conf] >= 4) {
    /* TC0, TC1を初期化 */
    TCCR0A = 0xa3;		/* Fast PWM mode(3) */
    TCCR0B = 0x03;		/* 1/64 clock */
    OCR0A = 0;			/* for channel 0 */
    OCR0B = 0;			/* for channel 1 */
    TCCR1A = 0xa1;		/* mode = 5 */
    TCCR1B = 0x0b;
    OCR1A = 0;
    OCR1B = 0;
  }
  if (AOPorts[conf] >= 6) {
    /* TC2を初期化 */
    TCCR2A = 0xa3;
    TCCR2B = 0x03;
    OCR2A = 0;			/* for channel 4 */
    OCR2B = 0;			/* for channel 5 */
  }

}


/* Digital Output */
void gioSetDOPortAll(uint16_t vals) {
  uchar i;
  uchar maxPort = DOPorts[config];
  for (i = 0 ; i < maxPort ; i++) {
    gioSetDOPort(i, vals & 1);
    vals = vals >> 1;
  }
}

void gioSetDOPort(uchar portnum, uchar val) {
  struct portInfo *pip = portMapper[config].Dout;
  struct portInfo pi = pip[portnum];
  /*  struct portInfo pi = (portMapper[config].Dout)[portnum];*/

  setPortBit(pi.port, pi.bit, val);
}

/* Digital Input */
uint16_t gioGetDIPortAll() {
  uint16_t rtn = 0;
  char i;
  uchar maxPort = DIPorts[config];
  for (i = maxPort - 1 ; i >= 0 ; i--) {
    rtn = (rtn << 1) + gioGetDIPort(i);
  }
  return rtn;
}

uchar gioGetDIPort(uchar portnum) {
  struct portInfo *pip = portMapper[config].Din;
  struct portInfo pi = pip[portnum];

  return getPortBit(pi.port, pi.bit);
}

/* Analog Output with PWM */
void gioSetAOPort(uchar portnum, uchar val) {
  struct portInfo *pip = portMapper[config].Aout;
  struct portInfo pi = pip[portnum];

  *(pi.port) = val;
}

void gioSetAOPortAll(uchar *s) {
  char i;
  uchar maxPort = AOPorts[config];
  uchar val;
  for (i = 0 ; i < maxPort ; i++) {
    val = hexToUchar2(s);
    gioSetAOPort(i, val);
    s += 2;
  }
}

#ifdef MODEL_GINGER
/* polling for check sw */
uchar gioPollSw() {
  static uchar pre = 0;
  uchar val = gioGetSW();
  if (pre == 0 && val == 1) {
    pre = 1;
    return 2;			/* sw off */
  }
  if (pre == 1 && val == 0){
    pre = 0;
    return 1;			/* sw on */
  }
  return 0;			/* same, no action */
}

#endif
/* Analog Input with ADC */

uchar gioADchannel() {
  return AIPorts[config];
}

uchar gioAOchannel() {
  return AOPorts[config];
}

uchar gioDIchannel() {
  return DIPorts[config];
}


void wait( unsigned char time) {
  unsigned char j,k;
  for (j = 0; j < time; j++)
    for (k = 0; k < 250; k++)
      ;
}

/* EOF */
